/* Copyright (c) 2015, 2017, 2020 The Linux Foundation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *     * Neither the name of The Linux Foundation, nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __IDATAITEMCORE_H__
#define __IDATAITEMCORE_H__

#include <string>
#include <DataItemId.h>

namespace loc_core {

using namespace std;

/**
 * @brief IDataItemCore interface.
 * @details IDataItemCore interface.
 *
 */
class IDataItemCore {
public:
    /**
     * @brief Gets Data item id.
     * @details Gets Data item id.
     * @return Data item id.
     */
    inline DataItemId getId() { return mId; }

    /**
     * @brief Stringify.
     * @details Stringify.
     *
     * @param valueStr Reference to string.
     */
    virtual void stringify (string & valueStr) = 0;

    /**
     * @brief copy.
     * @details copy.
     *
     * @param src Where to copy from.
     * @param dataItemCopied Boolean flag indicated whether or not copied.
     *
     * @return Zero for success or non zero for failure.
     */
    virtual int32_t copyFrom(IDataItemCore * src) = 0;

    /**
     * @brief Destructor.
     * @details Destructor.
     */
    virtual ~IDataItemCore () {}
protected:
    DataItemId mId = INVALID_DATA_ITEM_ID;
};

} // namespace loc_core

#endif // __IDATAITEMCORE_H__
